/*
 * shadowircd: an advanced IRC daemon.
 * umodes.h: part of new usermodes system
 *
 * $Id: umodes.h 191 2005-04-11 19:35:42Z nenolod $
 */

/* nice flashy bitfield math, care of asuffield and dancer-ircd.
 * adopted where necessary 05/12/04 by nenolod.
 *
 * some parts rewritten for efficiency too.
 */
#ifndef INCLUDED_UMODES_H
#define INCLUDED_UMODES_H

typedef struct {
	int mode;
	char letter;
	int operonly;
	int untouchable; /* for SSL modes. */
} FLAG_ITEM;

struct bitfield_lookup_t {
	unsigned int field;
	u_int32_t bit;
};

/* craq++ follows */
#ifdef THIS_IS_UMODES_C
# define Extern /* extern */
#else
# define Extern extern
#endif

/* This is initialised in init_umodes() in s_user.c, based on the value of 
 * BITFIELD_SIZE
 */
extern struct bitfield_lookup_t bitfield_lookup[];
extern char umode_list[];

/* the BITFIELD_SIZE define is used to create the bitfield. */
#define BITFIELD_SIZE 64

/* one bit is always needed for invalid modes as a seperator */
#define MAX_UMODE_COUNT (BITFIELD_SIZE)

/* do not change this. ever. not without changing the macros -- they are
 * hardcoded for two 32-bit bitfields.
 */
typedef u_int32_t user_modes[2];

#define SetBit(f,b)   (((f)[bitfield_lookup[b].field]) |=  bitfield_lookup[b].bit)
#define ClearBit(f,b) (((f)[bitfield_lookup[b].field]) &= ~bitfield_lookup[b].bit)
#define TestBit(f,b)  (((f)[bitfield_lookup[b].field]) &  bitfield_lookup[b].bit)

/* borrowed from asuffield's dancer-ircd. --nenolod */
#define CopyUmodes(d,s) (((d)[0] = (s)[0]), ((d)[1] = (s)[1]))
#define AndUmodes(d,a,b) (((d)[0] = (a)[0] & (b)[0]), ((d)[1] = (a)[1] & (b)[1]))
#define OrUmodes(d,a,b) (((d)[0] = (a)[0] | (b)[0]), ((d)[1] = (a)[1] | (b)[1]))
#define AndNotUmodes(d,a,b) (((d)[0] = (a)[0] & ~(b)[0]), ((d)[1] = (a)[1] & ~(b)[1]))
#define NotUmodes(d,s) (((d)[0] = ~s[0]), ((d)[1] = ~s[1]))
#define AnyBits(d) ((d)[0] | (d)[1])
#define SameBits(a,b) (((a)[0] == (b)[0]) && ((a)[1] == (b)[1]))

#define ClearBitfield(b) (((b)[0] = 0), ((b)[1] = 0))

/* These are usually used */
#define HasUmode(c,m) (TestBit((c)->umodes,m))
#define SetUmode(c,m) (SetBit((c)->umodes,m))
#define ClearUmode(c,m) (ClearBit((c)->umodes,m))

/* usermodes. now extern int. the core umodes live in umodes.c right now,
 * and are determined by the return value of register_umode():
 * 
 * int UMODE_SERVNOTICE = register_umode('s', 0, 0, 0);
 *
 * Which puts UMODE_SERVNOTICE into the dlink_list, and then builds a table
 * from it.
 */
Extern int UMODE_SERVNOTICE;  /* server notices     */                     /* +s */
Extern int UMODE_CCONN;       /* Client Connections */                     /* +c */
Extern int UMODE_REJ;         /* Bot Rejections */                         /* +j */
Extern int UMODE_SKILL;       /* Server Killed */                          /* +k */
Extern int UMODE_FULL;        /* Full messages */                          /* +f */
Extern int UMODE_SPY;         /* see STATS / LINKS */                      /* +y */
Extern int UMODE_DEBUG;       /* 'debugging' info */                       /* +d */
Extern int UMODE_NCHANGE;     /* Nick change notice */                     /* +n */
Extern int UMODE_WALLOP;      /* send wallops to them */                   /* +w */
Extern int UMODE_OPERWALL;    /* Operwalls */                              /* +z */
Extern int UMODE_INVISIBLE;   /* makes user invisible */                   /* +i */
Extern int UMODE_BOTS;        /* shows bots */                             /* +b */
Extern int UMODE_EXTERNAL;    /* show servers introduced and splitting */  /* +X */
Extern int UMODE_CALLERID;    /* block unless caller id's */               /* +g */
Extern int UMODE_UNAUTH;      /* show unauth connects here */              /* +u */
Extern int UMODE_LOCOPS;      /* show locops */                            /* +l */
Extern int UMODE_OPER;        /* Operator */                               /* +o */
Extern int UMODE_ADMIN;       /* Admin on server */                        /* +a */
Extern int UMODE_IDENTIFY;    /* Identified. */                            /* +e */
Extern int UMODE_HIDEOPER;    /* Hide operator status. */                  /* +h */
Extern int UMODE_CLOAK;       /* Usercloak. */                             /* +v */
Extern int UMODE_PROTECTED;   /* user cannot be kicked from channels */    /* +q */
Extern int UMODE_BLOCKINVITE; /* Block Invites. */                         /* +I */
Extern int UMODE_PMFILTER;    /* Allow only registered users to PM */      /* +E */
Extern int UMODE_HELPOP;      /* Help operator. */                         /* +H */
Extern int UMODE_SVSOPER;     /* Services operator. */                     /* +O */
Extern int UMODE_SVSADMIN;    /* Services admin. */                        /* +A */
Extern int UMODE_SVSROOT;     /* Services root. */                         /* +R */
Extern int UMODE_SERVICE;     /* Network service. */                       /* +S */
Extern int UMODE_SECURE;      /* client is using SSL */                    /* +Z */
Extern int UMODE_DEAF;        /* User is deaf. */                          /* +D */

/* vanity flags */
Extern int UMODE_NETADMIN;    /* network administrator */                  /* +N */
Extern int UMODE_TECHADMIN;   /* technical administrator */                /* +T */

Extern int UMODE_NOCOLOUR;    /* block message colours */                  /* +C */
Extern int UMODE_SENSITIVE;   /* user has "sensitive ears" */              /* +G */

Extern int UMODE_ROUTING;     /* user is on routing team */                /* +L */

Extern int UMODE_KILLPROT;    /* user cannot be killed, except by NA */    /* +K */
Extern int UMODE_WANTSWHOIS;  /* Whois notifications */                    /* +W */

/* macros for tests... cleaner code */
#define IsInvisible(x)          (HasUmode(x, UMODE_INVISIBLE))
#define IsSetCallerId(x)        (HasUmode(x, UMODE_CALLERID))
#define IsPMFiltered(x)         (HasUmode(x, UMODE_PMFILTER))
#define IsIdentified(x)         (HasUmode(x, UMODE_IDENTIFY))
#define IsBlockInvite(x)        (HasUmode(x, UMODE_BLOCKINVITE))
#define IsDeaf(x)               (HasUmode(x, UMODE_DEAF))
#define IsSensitive(x)          (HasUmode(x, UMODE_SENSITIVE))
#define BlockColour(x)          (HasUmode(x, UMODE_NOCOLOUR))
#define IsSecure(x)             (HasUmode(x, UMODE_SECURE))
#define IsCloaked(x)            (HasUmode(x, UMODE_CLOAK))
#define IsHiding(x)             (HasUmode(x, UMODE_HIDEOPER))

#define IsOper(x)               (HasUmode(x, UMODE_OPER))
#define IsAdmin(x)              (HasUmode(x, UMODE_ADMIN))
#define IsHelpop(x)             (HasUmode(x, UMODE_HELPOP))
#define IsNetadmin(x)           (HasUmode(x, UMODE_NETADMIN))
#define IsTechadmin(x)          (HasUmode(x, UMODE_TECHADMIN))
#define IsServicesRoot(x)       (HasUmode(x, UMODE_SVSROOT))
#define IsServicesAdmin(x)      (HasUmode(x, UMODE_SVSADMIN))
#define IsServicesOper(x)       (HasUmode(x, UMODE_SVSOPER))
#define IsService(x)            (HasUmode(x, UMODE_SERVICE))
#define IsRouting(x)            (HasUmode(x, UMODE_ROUTING))

#define SendRejections(x)       (HasUmode(x, UMODE_REJ))
#define SendNotices(x)          (HasUmode(x, UMODE_SERVNOTICE))
#define SendConnects(x)         (HasUmode(x, UMODE_CCONN))
#define SendServerKills(x)      (HasUmode(x, UMODE_SKILL))
#define SendFullNotice(x)       (HasUmode(x, UMODE_FULL))
#define SendDebugNotice(x)      (HasUmode(x, UMODE_DEBUG))
#define SendNickChange(x)       (HasUmode(x, UMODE_NCHANGE))
#define SendWallops(x)          (HasUmode(x, UMODE_WALLOP))
#define SendOperwall(x)         (HasUmode(x, UMODE_OPERWALL))
#define SendBotNotices(x)       (HasUmode(x, UMODE_BOTS))
#define SendExternalNotices(x)  (HasUmode(x, UMODE_EXTERNAL))
#define SendLocops(x)           (HasUmode(x, UMODE_LOCOPS))
#define SendUnauthorized(x)     (HasUmode(x, UMODE_UNAUTH))

extern FLAG_ITEM user_mode_table[];
extern void  init_umodes(void);
extern char *umodes_as_string(user_modes *);
extern void umodes_from_string(user_modes *, char *);
extern char *umode_difference(user_modes *, user_modes *);
extern user_modes *build_umodes(user_modes *, int, ...);
extern int available_slot;
extern int register_umode(char, int, int, int);
extern void setup_umodesys(void);

#endif

