/*
 *  shadowircd: an advanced Internet Relay Chat Daemon(ircd).
 *  m_error.c: Handles error messages from the other end.
 *
 *  Copyright (C) 2005 by the past and present ircd coders, and others.
 *
 *  This program is free software; see LICENSE in the root of this
 *  source tree for details.
 *
 *  $Id: m_error.c 191 2005-04-11 19:35:42Z nenolod $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"		/* FALSE */
#include "ircd.h"
#include "send.h"
#include "msg.h"
#include "memory.h"
#include "s_log.h"

struct Message error_msgtab = {
	"ERROR", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0,
	{m_error, m_ignore, ms_error, m_ignore, m_ignore}
};

/*
 * Note: At least at protocol level ERROR has only one parameter,
 * although this is called internally from other functions
 * --msa
 *
 *      parv[0] = sender prefix
 *      parv[*] = parameters
 */
void
m_error(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	const char *para;

	para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";

	ilog(L_ERROR, "Received ERROR message from %s: %s", source_p->name, para);

	if(client_p == source_p)
	{
		sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s -- %s",
				     get_client_name(client_p, HIDE_IP), para);
		sendto_realops_flags(UMODE_ALL, L_OPER, "ERROR :from %s -- %s",
				     get_client_name(client_p, MASK_IP), para);
	}
	else
	{
		sendto_realops_flags(UMODE_ALL, L_OPER, "ERROR :from %s via %s -- %s",
				     source_p->name, get_client_name(client_p, MASK_IP), para);
		sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s via %s -- %s",
				     source_p->name, get_client_name(client_p, HIDE_IP), para);
	}

	if(MyClient(source_p))
		exit_client(client_p, source_p, source_p, "ERROR");
}

void
ms_error(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	const char *para;

	para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";

	ilog(L_ERROR, "Received ERROR message from %s: %s", source_p->name, para);

	if(client_p == source_p)
		sendto_realops_flags(UMODE_ALL, L_ALL, "ERROR :from %s -- %s",
				     get_client_name(client_p, MASK_IP), para);
	else
		sendto_realops_flags(UMODE_ALL, L_ALL, "ERROR :from %s via %s -- %s",
				     source_p->name, get_client_name(client_p, MASK_IP), para);

	if(MyClient(source_p))
		exit_client(client_p, source_p, source_p, "ERROR");
}
