/*
 * shadowircd: an advanced IRC daemon.
 * umodes.c: New usermode system, derived from dancer-ircd's usermode code.
 *
 * $Id: umodes.c 191 2005-04-11 19:35:42Z nenolod $
 */
#define THIS_IS_UMODES_C /* craq++ */
#include "stdinc.h"
#include "tools.h"
#include "list.h"
#include "memory.h"
#include "s_user.h"
#include "umodes.h"

FLAG_ITEM user_mode_table[256] = {
	{0, 0, 0}
};

extern void mode_sort(char *);

int user_modes_from_c_to_bitmask[256];
char umode_list[256];
int available_slot = 0;

struct bitfield_lookup_t bitfield_lookup[BITFIELD_SIZE];

dlink_list user_mode_list; /* turned into the table. */

void
init_umodes(void)
{
	unsigned int i, field, bit;
	char *p = umode_list;
	available_slot = 0;
	/* Fill out the reverse umode map */
	for(i = 0; user_mode_table[i].letter; i++)
	{
		user_modes_from_c_to_bitmask[(unsigned char) user_mode_table[i].letter] =
			user_mode_table[i].mode;
		*p++ = user_mode_table[i].letter;
		available_slot++;
	}
	*p = '\0';
	/* Fill out the bitfield map */
	bit = 1;
	field = 0;
	for(i = 0; i < BITFIELD_SIZE; i++)
	{
		bitfield_lookup[i].bit = bit;
		bitfield_lookup[i].field = field;
		if(bit == 0x80000000)
		{
			field++;
			bit = 1;
		}
		else
			bit = bit << 1;
	}

	mode_sort(umode_list);
}

char *
umodes_as_string(user_modes * flags)
{
	static char flags_out[MAX_UMODE_COUNT];
	char *flags_ptr;
	int i;

	flags_ptr = flags_out;
	*flags_ptr = '\0';

	for(i = 0; user_mode_table[i].letter; i++)
	{
		if(TestBit(*flags, user_mode_table[i].mode))
			*flags_ptr++ = user_mode_table[i].letter;
	}
	*flags_ptr = '\0';
	return (flags_out);
}

void
umodes_from_string(user_modes * u, char *flags)
{
	if(!flags)
		return;

	for(; *flags; flags++)
		if(user_modes_from_c_to_bitmask[(unsigned char) *flags])
			SetBit((*u), user_modes_from_c_to_bitmask[(unsigned char) *flags]);
}

user_modes *
build_umodes(user_modes * u, int modes, ...)
{
	va_list args;
	int i;

	va_start(args, modes);
	for(i = 0; i < modes; i++)
		SetBit(*u, user_mode_table[va_arg(args, unsigned int)].mode);
	va_end(args);
	return u;
}

char *
umode_difference(user_modes * old_modes, user_modes * new_modes)
{
	static char buf[MAX_UMODE_COUNT + 3];
	int i, flag;
	char *p = buf;
	char t = '=';

	/* Do this twice because we don't want +a-b+c-d and so on, we
	 * want -bd+ac
	 */
	for(i = 0; user_mode_table[i].letter; i++)
	{
		flag = user_mode_table[i].mode;

		if(TestBit(*old_modes, flag) && !TestBit(*new_modes, flag))
		{
			if(t != '-')
				t = *p++ = '-';
			*p++ = user_mode_table[i].letter;
		}
	}

	for(i = 0; user_mode_table[i].letter; i++)
	{
		flag = user_mode_table[i].mode;

		if(!TestBit(*old_modes, flag) && TestBit(*new_modes, flag))
		{
			if(t != '+')
				t = *p++ = '+';
			*p++ = user_mode_table[i].letter;
		}
	}
	*p = '\0';
	return buf;
}

/*
 * bootstrap_umodes: craq++
 *
 * input        - none
 * output       - none
 * side effects - dlink_list user_mode_list is turned into
 *                user_mode_table, then umodes is initialized.
 */
static void
bootstrap_umodes(void)
{
	dlink_node *ptr;
	available_slot = 0;

	DLINK_FOREACH(ptr, user_mode_list.head)
	{
		FLAG_ITEM *modeptr = ptr->data;

		user_mode_table[available_slot].letter = modeptr->letter;
		user_mode_table[available_slot].mode = modeptr->mode;
		user_mode_table[available_slot].operonly = modeptr->operonly;
		user_mode_table[available_slot].untouchable = modeptr->untouchable;

		available_slot++;
	}

	init_umodes();
}

/*
 * register_umode: adds a umode to the ircd :)
 *
 * input        - letter, slot, operonly (true or false)
 * output       - none
 * side effects - umode is added to the umode table
 */
int
register_umode(char letter, int operonly, int untouchable, int rebuild)
{
	dlink_node *ptr = make_dlink_node();
	FLAG_ITEM *modeptr = MyMalloc(sizeof(FLAG_ITEM));

	modeptr->letter = letter;
	modeptr->mode = available_slot;
	modeptr->operonly = operonly;
	modeptr->untouchable = untouchable;

	dlinkAddTail(modeptr, ptr, &user_mode_list);

	available_slot++;

	if(rebuild)
		bootstrap_umodes();

	return modeptr->mode;
}

void
setup_umodesys(void)
{
	UMODE_SERVNOTICE  = register_umode('s', 0, 0, 0);
	UMODE_CCONN       = register_umode('c', 1, 0, 0);
	UMODE_REJ         = register_umode('r', 1, 0, 0);
	UMODE_SKILL       = register_umode('k', 1, 0, 0);
	UMODE_FULL        = register_umode('f', 1, 0, 0);
	UMODE_SPY         = register_umode('y', 1, 0, 0);
	UMODE_DEBUG       = register_umode('d', 1, 0, 0);
	UMODE_NCHANGE     = register_umode('n', 1, 0, 0);
	UMODE_WALLOP      = register_umode('w', 0, 0, 0);
	UMODE_OPERWALL    = register_umode('z', 1, 0, 0);
	UMODE_INVISIBLE   = register_umode('i', 0, 0, 0);
	UMODE_BOTS        = register_umode('b', 1, 0, 0);
	UMODE_EXTERNAL    = register_umode('x', 1, 0, 0);
	UMODE_CALLERID    = register_umode('g', 0, 0, 0);
	UMODE_UNAUTH      = register_umode('u', 1, 0, 0);
	UMODE_LOCOPS      = register_umode('l', 1, 0, 0);
	UMODE_OPER        = register_umode('o', 1, 0, 0);
	UMODE_ADMIN       = register_umode('a', 1, 0, 0);
	UMODE_IDENTIFY    = register_umode('e', 1, 0, 0);
	UMODE_HIDEOPER    = register_umode('h', 1, 0, 0);
	UMODE_CLOAK       = register_umode('v', 0, 0, 0);
	UMODE_PROTECTED   = register_umode('q', 1, 0, 0);
	UMODE_BLOCKINVITE = register_umode('I', 0, 0, 0);
	UMODE_PMFILTER    = register_umode('E', 0, 0, 0);
	UMODE_HELPOP      = register_umode('H', 1, 0, 0);
	UMODE_SVSOPER     = register_umode('O', 1, 0, 0);
	UMODE_SVSADMIN    = register_umode('A', 1, 0, 0);
	UMODE_SVSROOT     = register_umode('R', 1, 0, 0);
	UMODE_SERVICE     = register_umode('S', 1, 1, 0);
	UMODE_SECURE      = register_umode('Z', 1, 1, 0);
	UMODE_DEAF        = register_umode('D', 1, 0, 0);
	UMODE_NETADMIN    = register_umode('N', 1, 0, 0);
	UMODE_TECHADMIN   = register_umode('T', 1, 0, 0);
	UMODE_NOCOLOUR    = register_umode('C', 0, 0, 0);
	UMODE_SENSITIVE   = register_umode('G', 0, 0, 0);
	UMODE_ROUTING     = register_umode('L', 1, 0, 0);
	UMODE_KILLPROT    = register_umode('K', 1, 0, 0);
	UMODE_WANTSWHOIS  = register_umode('W', 1, 0, 1);
}

